@@ -14,9 +14,13 @@ __all__ = ['MapTableFile',
            'MTIndex',
            'MTContaminant',
            'MTSediment']
-import os
 
 from future.utils import iteritems
+import logging
+import numpy as np
+import pandas as pd
+import os
+from osgeo import gdalconst
 from sqlalchemy import ForeignKey, Column
 from sqlalchemy.types import Integer, Float, String
 from sqlalchemy.orm import relationship
@@ -27,8 +31,11 @@ from ..base.file_base import GsshaPyFileObjectBase
 from .idx import IndexMap
 from ..lib import parsetools as pt
 from ..lib import cmt_chunk as mtc
+from ..lib.grid_tools import resample_grid
 from ..lib.parsetools import valueReadPreprocessor as vrp, valueWritePreprocessor as vwp
 
+log = logging.getLogger(__name__)
+
 
 class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
     """
@@ -58,15 +65,19 @@ class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
     fileExtension = Column(String, default='cmt')  #: STRING
 
     # Relationship Properties
-    indexMaps = relationship('IndexMap', back_populates='mapTableFile')  #: RELATIONSHIP
+    indexMaps = relationship('IndexMap', back_populates='mapTableFile', lazy='dynamic')  #: RELATIONSHIP
     mapTables = relationship('MapTable', back_populates='mapTableFile')  #: RELATIONSHIP
     projectFile = relationship('ProjectFile', uselist=False, back_populates='mapTableFile')  #: RELATIONSHIP
 
-    def __init__(self):
+    def __init__(self, project_file=None):
         """
         Constructor
         """
         GsshaPyFileObjectBase.__init__(self)
+        # add to project_file
+        self.projectFile = project_file
+        # Set file extension property
+        self.fileExtension = 'cmt'
 
     def _read(self, directory, filename, session, path, name, extension,
               spatial=False, spatialReferenceID=4236, replaceParamFile=None,
@@ -160,14 +171,11 @@ class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
 
                 contaminants = sorted(contaminantList, key=lambda x: (x.indexMap.name, x.name))
 
-        # Derive a set of unique MTIndexMap objects
-        indexMaps = self.indexMaps
-
         # Write first line to file
         openFile.write('GSSHA_INDEX_MAP_TABLES\n')
 
         # Write list of index maps
-        for indexMap in indexMaps:
+        for indexMap in self.indexMaps:
             # Write to map table file
             openFile.write('INDEX_MAP%s"%s" "%s"\n' % (' ' * 16, indexMap.filename, indexMap.name))
 
@@ -199,6 +207,18 @@ class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
         """
         return session.query(MapTable).filter(MapTable.mapTableFile == self).order_by(MapTable.name).all()
 
+    def deleteMapTable(self, name, session):
+        """
+        Remove duplicate map table if it exists
+        """
+
+        duplicate_map_tables = session.query(MapTable).filter(MapTable.mapTableFile == self).filter(MapTable.name == name).all()
+        for duplicate_map_table in duplicate_map_tables:
+            if duplicate_map_table.indexMap:
+                session.delete(duplicate_map_table.indexMap)
+            session.delete(duplicate_map_table)
+            session.commit()
+
     def _createGsshaPyObjects(self, mapTables, indexMaps, replaceParamFile, directory, session, spatial, spatialReferenceID):
         """
         Create GSSHAPY Mapping Table ORM Objects Method
@@ -271,9 +291,9 @@ class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
                     self._createValueObjects(mt['valueList'], mt['varList'], mapTable, indexMap, None, replaceParamFile)
 
             except KeyError:
-                print(('INFO: Index Map "%s" for Mapping Table "%s" not found in list of index maps in the mapping '
-                       'table file. The Mapping Table was not read into the database.') % (
-                       mt['indexMapName'], mt['name']))
+                log.info(('Index Map "%s" for Mapping Table "%s" not found in list of index maps in the mapping '
+                          'table file. The Mapping Table was not read into the database.') % (
+                          mt['indexMapName'], mt['name']))
 
     def _createValueObjects(self, valueList, varList, mapTable, indexMap, contaminant, replaceParamFile):
         """
@@ -328,7 +348,7 @@ class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
                                          spatial=spatial,
                                          spatialReferenceID=spatialReferenceID)
             except:
-                print('WARNING: Attempted to read Contaminant Transport Output file {0}, but failed.'.format(chanFile))
+                log.warning('Attempted to read Contaminant Transport Output file {0}, but failed.'.format(chanFile))
 
     def _writeMapTable(self, session, fileObject, mapTable, replaceParamFile):
         """
@@ -558,6 +578,146 @@ class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
 
         return outputFilename
 
+    def addRoughnessMapFromLandUse(self, name,
+                                         session,
+                                         land_use_grid,
+                                         land_use_to_roughness_table=None,
+                                         land_use_grid_id=None,
+
+                                         ):
+        '''
+        Adds a roughness map from land use file
+
+        Example::
+
+            from gsshapy.orm import ProjectFile
+            from gsshapy.lib import db_tools as dbt
+
+            from os import path, chdir
+
+            gssha_directory = '/gsshapy/tests/grid_standard/gssha_project'
+            land_use_grid = 'LC_5min_global_2012.tif'
+            land_use_to_roughness_table = ''/gsshapy/gridtogssha/land_cover/land_cover_glcf_modis.txt'
+
+            chdir(gssha_directory)
+
+            # Create Test DB
+            sqlalchemy_url, sql_engine = dbt.init_sqlite_memory()
+
+            # Create DB Sessions
+            db_session = dbt.create_session(sqlalchemy_url, sql_engine)
+
+            # Instantiate GSSHAPY object for reading to database
+            project_manager = ProjectFile()
+
+            # Call read method
+            project_manager.readInput(directory=gssha_directory,
+                                      projectFileName='grid_standard.prj',
+                                      session=db_session)
+
+            project_manager.mapTableFile.addRoughnessMapFromLandUse("roughness",
+                                                                    db_session,
+                                                                    land_use_to_roughness_table,
+                                                                    land_use_grid,
+                                                                    )
+            # WRITE OUT UPDATED GSSHA PROJECT FILE
+            project_manager.writeInput(session=db_session,
+                                       directory=gssha_directory,
+                                       name='grid_standard')
+
+        '''
+        LAND_USE_GRID_TABLES = {
+                                 'nga'  : 'land_cover_nga.txt',
+                                 'glcf' : 'land_cover_glcf_modis.txt',
+                                 'nlcd' : 'land_cover_nlcd.txt',
+                                }
+
+        # read in table
+        if isinstance(land_use_to_roughness_table, pd.DataFrame):
+            df = land_use_to_roughness_table
+        else:
+            if land_use_to_roughness_table is None:
+                if land_use_grid_id is None:
+                    raise ValueError("Must have land_use_to_roughness_table or land_use_grid_id set ...")
+
+                land_use_to_roughness_table = os.path.join(os.path.dirname(os.path.realpath(__file__)),
+                                                           '..', 'grid',
+                                                           'land_cover',
+                                                           LAND_USE_GRID_TABLES[land_use_grid_id])
+
+            df = pd.read_table(land_use_to_roughness_table, delim_whitespace=True,
+                               header=None, skiprows=1,
+                               names=('id', 'description', 'roughness'),
+                               dtype={'id':'int', 'description':'str', 'roughness':'float'},
+                               )
+
+
+        # resample land use grid to gssha grid
+        land_use_resampled = resample_grid(land_use_grid,
+                                           self.projectFile.getGrid(),
+                                           resample_method=gdalconst.GRA_NearestNeighbour,
+                                           as_gdal_grid=True)
+
+        unique_land_use_ids = np.unique(land_use_resampled.np_array())
+
+        #only add ids in index map subset
+        df = df[df['id'].isin(unique_land_use_ids)]
+
+        # make sure all needed land use IDs exist
+        for land_use_id in unique_land_use_ids:
+            if land_use_id not in df.index:
+                raise IndexError("Land use ID {0} not found in table.".format(land_use_id))
+
+        # delete duplicate/old tables with same name if they exist
+        self.deleteMapTable("ROUGHNESS", session)
+
+        # get num ids
+        mapTable = MapTable(name="ROUGHNESS",
+                            numIDs=len(df.index),
+                            maxNumCells=0,
+                            numSed=0,
+                            numContam=0)
+
+        # Create GSSHAPY IndexMap object from result object
+        indexMap = IndexMap(name=name)
+        indexMap.mapTableFile = self
+        mapTable.indexMap = indexMap
+
+        # Associate MapTable with this MapTableFile and IndexMaps
+        mapTable.mapTableFile = self
+        # add values to table
+        for row in df.itertuples():
+            idx = MTIndex(row.id, row.description, '')
+            idx.indexMap = indexMap
+            val = MTValue('ROUGH', row.roughness)
+            val.index = idx
+            val.mapTable = mapTable
+
+
+        project_path = ""
+        proj_path_card = self.projectFile.getCard('PROJECT_PATH')
+        if proj_path_card:
+            project_path = proj_path_card.value.strip('"').strip("'")
+
+        # remove MANNING_N card becasue it is mutually exclusive
+        manningn_card = self.projectFile.getCard('MANNING_N')
+        if manningn_card:
+            session.delete(manningn_card)
+            session.commit()
+
+        # add path to file
+        mapTable.indexMap.filename = os.path.join(project_path,
+                                                  '{0}.idx'.format(name))
+
+        # write file
+        land_use_resampled.to_grass_ascii(mapTable.indexMap.filename,
+                                          print_nodata=False)
+
+        # update project card
+        if not self.projectFile.getCard('MAPPING_TABLE'):
+            self.projectFile.setCard('MAPPING_TABLE',
+                                     '{0}.cmt'.format(self.projectFile.name),
+                                     add_quotes=True)
 
 class MapTable(DeclarativeBase):
     """
